@@ -22,45 +22,6 @@ void osp_croak(const char* pat, ...)
 
 /* CCov: on */
 
-/*--------------------------------------------- osp_bridge */
-
-osp_bridge::osp_bridge()
-{
-  // optimize XXX
-  mysv_lock(osp_thr::TXGV);
-  assert(AvFILL(osp_thr::TXStack) >= 0);
-  SV *txref = *av_fetch(osp_thr::TXStack, AvFILL(osp_thr::TXStack), 0);
-  assert(SvROK(txref));
-  txsv = SvRV(txref);
-  SvREFCNT_inc(txsv);
-
-  osp_txn *txn = (osp_txn*) SvIV(txsv);
-  if (!txn) {
-    // should be impossible XXX
-    warn("array:");
-    mysv_dump((SV*)osp_thr::TXStack);
-    warn("elem:");
-    mysv_dump(txref);
-    croak("Transaction null!  Race condition?");
-  }
-  next = txn->bridge_top;
-  txn->bridge_top = this;
-}
-
-osp_bridge::~osp_bridge()
-{
-//  assert(ready()); should be done in subclasses
-  DEBUG_bridge(this, warn("bridge(%p)->DESTROY", this));
-}
-void osp_bridge::release()
-{ croak("osp_bridge::release()"); }
-void osp_bridge::invalidate()
-{ croak("osp_bridge::invalidate()"); }
-int osp_bridge::invalid()
-{ croak("osp_bridge::invalid()"); return 1; }
-int osp_bridge::ready()
-{ croak("osp_bridge::ready()"); return 0; }
-
 /*--------------------------------------------- per-thread context */
 
 /* CCov: off */
@@ -113,7 +74,6 @@ SV *osp_thr::stargate=0;
 HV *osp_thr::CLASSLOAD;
 SV *osp_thr::TXGV;
 AV *osp_thr::TXStack;
-osp_bridge *osp_thr::bridge_top = 0;
 
 void osp_thr::boot()
 {
@@ -149,32 +109,16 @@ osp_thr::~osp_thr()
   // OS_END_FAULT_HANDLER;
 }
 
-void osp_thr::burn_bridge()
-{
-  osp_bridge *br = bridge_top;
-  bridge_top=0;
-  while (br) {
-    osp_bridge *nxt = br->next;
-    if (br->ready()) delete br;
-    else {
-      br->next = bridge_top;
-      bridge_top = br;
-    }
-    br = nxt;
-  }
-}
-
 /*--------------------------------------------- per-transaction context */
 
 osp_txn::osp_txn(os_transaction::transaction_type_enum _tt,
 		 os_transaction::transaction_scope_enum scope_in)
   : tt(_tt), ts(scope_in)
 {
-  osp_thr::burn_bridge();
-
-  mysv_lock(osp_thr::TXGV);
+//  serial = next_txn++;
   os = os_transaction::begin(tt, scope_in);
-  bridge_top = 0;
+  ring.next = &ring;
+  ring.prev = &ring;
 
   DEBUG_txn(warn("txn(%p)->new(%s, %s)", this,
 		 tt==os_transaction::read_only? "read":
@@ -211,11 +155,11 @@ void osp_txn::post_transaction()
 
   DEBUG_txn(warn("%p->post_transaction", this));
 
-  mysv_lock(osp_thr::TXGV);
-  osp_bridge *br = bridge_top;
-  while (br) {
-    br->invalidate();
-    br = br->next;
+  osp_bridge *br = (osp_bridge*) ring.next;
+  while (br != &ring) {
+    osp_bridge *next = (osp_bridge*) br->next;
+    br->leave_txn();
+    br = next;
   }
 }
 
@@ -232,9 +176,14 @@ int osp_txn::can_update(void *vptr)
   return db && db->is_writable();
 }
 
+int osp_txn::is_aborted()
+{
+  // returns true after commit XXX?
+  return !os || os->is_aborted();
+}
+
 void osp_txn::abort()
 {
-  mysv_lock(osp_thr::TXGV);
   os_transaction *copy = os;
   os = 0;
   if (!copy) return;
@@ -248,14 +197,11 @@ void osp_txn::abort()
 
 void osp_txn::commit()
 {
-  mysv_lock(osp_thr::TXGV);
   os_transaction *copy = os;
   os = 0;
   if (!copy) return;
   if (!copy->is_aborted()) {
-
-    // assert bridge is empty XXX
-
+    assert(ring.next == &ring);
     DEBUG_txn(warn("txn(%p)->commit", this));
     os_transaction::commit(copy);
   }
@@ -272,36 +218,101 @@ void osp_txn::pop()
   SvREADONLY_on(osp_thr::TXStack);
   assert(sv_isobject(myself) && (SvTYPE(SvRV(myself)) == SVt_PVMG));
   assert(this == (osp_txn*) SvIV((SV*) SvRV(myself)));
-  burn_bridge();
+  post_transaction();
   SvREFCNT_dec(myself);
 }
 
-void osp_txn::burn_bridge()
-{
-  dOSP;
-  int moved=0;
-  osp_bridge *br = bridge_top;
-  while (br) {
-    osp_bridge *nxt = br->next;
-    br->invalidate();
-    if (br->ready()) delete br;
-    else {
-      ++ moved;
-      br->next = osp->bridge_top;
-      osp->bridge_top = br;
-    }
-    br = nxt;
-  }
-  DEBUG_txn(warn("txn(%p)->burn_bridge", this));
-}
-
 void osp_txn::checkpoint()
 {
-  mysv_lock(osp_thr::TXGV);
   if (!os)
     croak("ObjStore: no transaction to checkpoint");
   if (os->is_aborted())
     croak("ObjStore: cannot checkpoint an aborted transaction");
+  assert(ring.next == &ring);
   os_transaction::checkpoint(os);
-  burn_bridge(); //XXX
 }
+
+/*--------------------------------------------- osp_bridge */
+
+osp_bridge::osp_bridge()
+{
+  detached = 0;
+  holding = 0;
+  manual_hold = 0;
+  next = prev = 0;
+  refs = 1;
+
+  // get transaction
+  assert(AvFILL(osp_thr::TXStack) >= 0);
+  SV *txref = *av_fetch(osp_thr::TXStack, AvFILL(osp_thr::TXStack), 0);
+  assert(SvROK(txref));
+  txsv = SvRV(txref);
+  SvREFCNT_inc(txsv);
+}
+
+osp_txn *osp_bridge::get_transaction()
+{
+  assert(txsv);
+  osp_txn *txn = (osp_txn*) SvIV(txsv);
+  if (!txn) {
+    // should be impossible XXX
+    warn("array:");
+    mysv_dump((SV*)osp_thr::TXStack);
+    croak("Transaction null!  Race condition?");
+  }
+  return txn;
+}
+
+void osp_bridge::enter_txn(osp_txn *txn)
+{
+  mysv_lock(osp_thr::TXGV);
+  // should be per-thread
+  assert(next==0);
+  next = txn->ring.next;
+  prev = &txn->ring;
+  prev->next = this;
+  next->prev = this;
+  ++refs;
+}
+
+void osp_bridge::leave_perl()
+{
+  DEBUG_bridge(this, warn("osp_bridge(%p)->leave_perl", this));
+  --refs;
+  leave_txn();
+}
+void osp_bridge::leave_txn()
+{
+  if (!detached) {
+    unref();
+    if (next) {
+      mysv_lock(osp_thr::TXGV);
+      // should be per-thread
+      --refs;
+      next->prev = prev;
+      prev->next = next;
+      next = prev = 0;
+    }
+    if (txsv) {
+      SvREFCNT_dec(txsv);
+      txsv = 0;
+    }
+    detached=1;
+  }
+  assert(refs >= 0);
+  if (refs == 0) delete this;
+}
+int osp_bridge::invalid()
+{ return detached; }
+osp_bridge::~osp_bridge()
+{
+  DEBUG_bridge(this, warn("bridge(%p)->DESTROY", this));
+}
+
+void osp_bridge::hold()
+{ croak("bridge::hold()"); }
+void osp_bridge::unref()
+{ croak("osp_bridge::unref()"); }
+int osp_bridge::is_weak()
+{ croak("osp_bridge::is_weak()"); return 0; }
+
